@@ -37,11 +37,11 @@
 #include "config.h"
 
 #include <memory>
+#include <rak/file_stat.h>
 
 #include "torrent/exceptions.h"
 #include "content.h"
 #include "data/file_meta.h"
-#include "data/file_stat.h"
 #include "data/chunk.h"
 #include "data/entry_list.h"
 #include "data/piece.h"
@@ -55,9 +55,6 @@ Content::Content() :
 
   m_entryList(new EntryList),
   m_rootDir(".") {
-
-  m_delayDownloadDone.set_slot(m_signalDownloadDone.make_slot());
-  m_delayDownloadDone.set_iterator(taskScheduler.end());
 }
 
 Content::~Content() {
@@ -91,22 +88,6 @@ Content::set_complete_hash(const std::string& hash) {
   m_hash = hash;
 }
 
-void
-Content::set_root_dir(const std::string& dir) {
-//   if (m_chunkTotal)
-//     throw internal_error("Tried to set root directory on Content that is open");
-
-  m_rootDir = dir;
-}
-
-// std::string
-// Content::get_hash(unsigned int index) {
-//   if (index >= m_chunkTotal)
-//     throw internal_error("Tried to get chunk hash from Content that is out of range");
-
-//   return m_hash.substr(index * 20, 20);
-// }
-
 uint32_t
 Content::chunk_index_size(uint32_t index) const {
   if (index + 1 != m_chunkTotal || m_entryList->bytes_size() % m_chunkSize == 0)
@@ -133,7 +114,6 @@ Content::is_valid_piece(const Piece& p) const {
     (uint32_t)p.get_index() < m_chunkTotal &&
 
     p.get_length() != 0 &&
-    p.get_length() < (1 << 17) &&
 
     // Make sure offset does not overflow 32 bits.
     p.get_offset() < (1 << 30) &&
@@ -153,7 +133,6 @@ Content::close() {
   clear();
 
   m_entryList->close();
-  taskScheduler.erase(&m_delayDownloadDone);
 }
 
 void
@@ -162,16 +141,14 @@ Content::clear() {
   m_bitfield = BitField(m_chunkTotal);
 }
 
-void
-Content::resize() {
-  m_entryList->resize_all();
-}
-
 bool
 Content::receive_chunk_hash(uint32_t index, const std::string& hash) {
-  if (index >= m_chunkTotal || m_bitfield[index] || m_completed >= m_chunkTotal)
+  if (index >= m_chunkTotal || m_completed >= m_chunkTotal)
     throw internal_error("Content::receive_chunk_hash(...) received an invalid index.");
 
+  if (m_bitfield[index])
+    throw internal_error("Content::receive_chunk_hash(...) received a chunk that has already been finished.");
+
   if (hash.empty() || std::memcmp(hash.c_str(), chunk_hash(index), 20) != 0)
     return false;
 
@@ -189,15 +166,6 @@ Content::receive_chunk_hash(uint32_t index, const std::string& hash) {
 
   std::for_each(first, last, std::mem_fun_ref(&EntryListNode::inc_completed));
 
-  // We delay emitting the signal to allow the delegator to clean
-  // up. If we do a straight call it would cause problems for
-  // clients that wish to close and reopen the torrent, as
-  // HashQueue, Delegator etc shouldn't be cleaned up at this point.
-  if (m_completed == m_chunkTotal &&
-      !m_delayDownloadDone.get_slot().blocked() &&
-      !taskScheduler.is_scheduled(&m_delayDownloadDone))
-    taskScheduler.insert(&m_delayDownloadDone, Timer::cache());
-
   return true;
 }
 
