@@ -38,11 +38,12 @@
 #define LIBTORRENT_DATA_HASH_QUEUE_H
 
 #include <string>
+#include <list>
 
 #include "hash_queue_node.h"
 #include "chunk_handle.h"
 
-#include "utils/task.h"
+#include "globals.h"
 
 namespace torrent {
 
@@ -71,6 +72,7 @@ public:
 
   void                push_back(ChunkHandle handle, SlotDone d, const std::string& id);
 
+  bool                has(const std::string& id);
   bool                has(const std::string& id, uint32_t index);
 
   void                remove(const std::string& id);
@@ -78,13 +80,13 @@ public:
 
   void                work();
 
-  uint32_t            get_read_ahead() const         { return m_readAhead; }
+  uint32_t            read_ahead() const             { return m_readAhead; }
   void                set_read_ahead(uint32_t bytes) { m_readAhead = bytes; }
 
-  uint32_t            get_interval() const           { return m_interval; }
+  uint32_t            interval() const               { return m_interval; }
   void                set_interval(uint32_t usec)    { m_interval = usec; }
 
-  uint32_t            get_max_tries() const          { return m_maxTries; }
+  uint32_t            max_tries() const              { return m_maxTries; }
   void                set_max_tries(uint32_t tries)  { m_maxTries = tries; }
 
 private:
@@ -93,7 +95,7 @@ private:
   inline void         willneed(int bytes);
 
   uint16_t            m_tries;
-  TaskItem            m_taskWork;
+  rak::priority_item  m_taskWork;
 
   uint32_t            m_readAhead;
   uint32_t            m_interval;
