@@ -38,10 +38,11 @@
 #define LIBTORRENT_DATA_HASH_TORRENT_H
 
 #include <string>
+#include <inttypes.h>
 #include <rak/functional.h>
+#include <rak/ranges.h>
 
 #include "data/chunk_handle.h"
-#include "utils/ranges.h"
 
 namespace torrent {
 
@@ -51,9 +52,11 @@ class DownloadWrapper;
 
 class HashTorrent {
 public:
-  typedef rak::mem_fn1<DownloadWrapper, void, ChunkHandle>           SlotCheckChunk;
-  typedef rak::mem_fn0<DownloadWrapper, void>                        SlotInitialHash;
-  typedef rak::mem_fn1<DownloadWrapper, void, const std::string&>    SlotStorageError;
+  typedef rak::ranges<uint32_t> Ranges;
+
+  typedef rak::mem_fun1<DownloadWrapper, void, ChunkHandle>           SlotCheckChunk;
+  typedef rak::mem_fun0<DownloadWrapper, void>                        SlotInitialHash;
+  typedef rak::mem_fun1<DownloadWrapper, void, const std::string&>    SlotStorageError;
   
   HashTorrent(ChunkList* c);
   ~HashTorrent() { clear(); }
@@ -66,6 +69,8 @@ public:
 
   Ranges&             ranges()                               { return m_ranges; }
 
+  uint32_t            position() const                       { return m_position; }
+
   HashQueue*          get_queue()                            { return m_queue; }
   void                set_queue(HashQueue* q)                { m_queue = q; }
 
