@@ -38,10 +38,8 @@
 
 #include <cstring>
 #include <limits>
-#include <sigc++/signal.h>
 
 #include "data/chunk_list.h"
-#include "parse/parse.h"
 #include "protocol/handshake_manager.h"
 #include "protocol/peer_connection_base.h"
 #include "torrent/exceptions.h"
@@ -49,7 +47,7 @@
 #include "tracker/tracker_manager.h"
 
 #include "choke_manager.h"
-#include "delegator_select.h"
+#include "chunk_selector.h"
 #include "download_main.h"
 
 namespace torrent {
@@ -58,33 +56,41 @@ DownloadMain::DownloadMain() :
   m_trackerManager(new TrackerManager()),
   m_chokeManager(new ChokeManager(&this->m_connectionList)),
   m_chunkList(new ChunkList),
+  m_chunkSelector(new ChunkSelector),
 
   m_started(false),
   m_isOpen(false),
   m_endgame(false),
 
+  m_uploadThrottle(NULL),
+  m_downloadThrottle(NULL),
+
   m_upRate(60),
   m_downRate(60) {
 
-  m_content.block_download_done(true);
+  m_delegator.slot_chunk_enable(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::using_index));
+  m_delegator.slot_chunk_disable(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::not_using_index));
+  m_delegator.slot_chunk_find(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::find));
+  m_delegator.slot_chunk_done(rak::make_mem_fun(this, &DownloadMain::receive_chunk_done));
+  m_delegator.slot_chunk_size(rak::make_mem_fun(&m_content, &Content::chunk_index_size));
 
-  m_taskTick.set_slot(sigc::mem_fun(*this, &DownloadMain::receive_tick));
-  m_taskTick.set_iterator(taskScheduler.end());
+  m_trackerManager->tracker_info()->slot_stat_down() = rak::make_mem_fun(&m_downRate, &Rate::total);
+  m_trackerManager->tracker_info()->slot_stat_up()   = rak::make_mem_fun(&m_upRate, &Rate::total);
+  m_trackerManager->tracker_info()->slot_stat_left() = rak::make_mem_fun(this, &DownloadMain::get_bytes_left);
 
-  m_taskTrackerRequest.set_slot(sigc::mem_fun(*this, &DownloadMain::receive_tracker_request));
-  m_taskTrackerRequest.set_iterator(taskScheduler.end());
+  m_taskTrackerRequest.set_slot(rak::mem_fn(this, &DownloadMain::receive_tracker_request));
 
-  m_chunkList->slot_create_chunk(rak::make_mem_fn(&m_content, &Content::create_chunk));
+  m_chunkList->slot_create_chunk(rak::make_mem_fun(&m_content, &Content::create_chunk));
 }
 
 DownloadMain::~DownloadMain() {
-  if (taskScheduler.is_scheduled(&m_taskTick) ||
-      taskScheduler.is_scheduled(&m_taskTrackerRequest))
-    throw internal_error("DownloadMain::~DownloadMain(): m_taskTick is scheduled");
+  if (m_taskTrackerRequest.is_queued())
+    throw internal_error("DownloadMain::~DownloadMain(): m_taskTrackerRequest is queued.");
 
   delete m_trackerManager;
   delete m_chokeManager;
   delete m_chunkList;
+  delete m_chunkSelector;
 }
 
 void
@@ -93,10 +99,8 @@ DownloadMain::open() {
     throw internal_error("Tried to open a download that is already open");
 
   m_content.open();
-  m_chunkList->resize(m_content.chunk_total());
-  m_bitfieldCounter.create(m_content.chunk_total());
 
-//   m_delegator.get_select().priority().add(Priority::NORMAL, 0, m_content.chunk_total());
+  m_chunkList->resize(m_content.chunk_total());
 
   m_isOpen = true;
 }
@@ -118,6 +122,7 @@ DownloadMain::close() {
   // Clear the chunklist last as it requires all referenced chunks to
   // be released.
   m_chunkList->clear();
+  m_chunkSelector->cleanup();
 }
 
 void DownloadMain::start() {
@@ -130,7 +135,6 @@ void DownloadMain::start() {
   m_started = true;
   m_lastConnectedSize = 0;
 
-  setup_start();
   m_trackerManager->send_start();
 
   receive_connect_peers();
@@ -161,7 +165,7 @@ DownloadMain::stop() {
     connection_list()->erase(connection_list()->front());
 
   m_trackerManager->send_stop();
-  setup_stop();
+  priority_queue_erase(&taskScheduler, &m_taskTrackerRequest);
 
   m_chunkList->sync_all();
 }
@@ -182,21 +186,12 @@ DownloadMain::get_bytes_left() const {
 void
 DownloadMain::update_endgame() {
   if (!m_endgame &&
-      m_content.chunks_completed() + m_delegator.get_chunks().size() + 0 >= m_content.chunk_total()) {
+      m_content.chunks_completed() + m_delegator.get_chunks().size() + 5 >= m_content.chunk_total()) {
     m_endgame = true;
     m_delegator.set_aggressive(true);
   }
 }
 
-void
-DownloadMain::receive_tick() {
-  taskScheduler.insert(&m_taskTick, (Timer::cache() + 30 * 1000000).round_seconds());
-  // Now done by the resource manager.
-  //m_chokeManager->cycle();
-
-  m_chunkList->sync_periodic();
-}
-
 void
 DownloadMain::receive_chunk_done(unsigned int index) {
   ChunkHandle handle = m_chunkList->get(index, false);
@@ -229,8 +224,8 @@ DownloadMain::receive_tracker_success() {
   if (!m_started)
     return;
 
-  taskScheduler.erase(&m_taskTrackerRequest);
-  taskScheduler.insert(&m_taskTrackerRequest, (Timer::cache() + 30 * 1000000).round_seconds());
+  priority_queue_erase(&taskScheduler, &m_taskTrackerRequest);
+  priority_queue_insert(&taskScheduler, &m_taskTrackerRequest, (cachedTime + 30 * 1000000).round_seconds());
 }
 
 void
@@ -238,9 +233,11 @@ DownloadMain::receive_tracker_request() {
   if (connection_list()->size() >= connection_list()->get_min_size())
     return;
 
-  if (connection_list()->size() >= m_lastConnectedSize + 10)
-    m_trackerManager->request_current();
-  else // Check to make sure we don't query after every connection to the primary tracker?
+  if (connection_list()->size() < m_lastConnectedSize + 10 ||
+      !m_trackerManager->request_current())
+    // Try the next tracker if we couldn't get enough peers from the
+    // current one, or if we have connected more than
+    // TrackerManager::max_num_request times.
     m_trackerManager->request_next();
 
   m_lastConnectedSize = connection_list()->size();
