@@ -48,6 +48,7 @@
 #include "data/hash_queue.h"
 #include "net/listen.h"
 #include "net/manager.h"
+#include "net/throttle_manager.h"
 
 #include "manager.h"
 #include "resource_manager.h"
@@ -64,21 +65,23 @@ Manager::Manager() :
   m_listen(new Listen),
   m_resourceManager(new ResourceManager),
 
+  m_uploadThrottle(new ThrottleManager),
+  m_downloadThrottle(new ThrottleManager),
+
   m_ticks(0) {
 
-  m_taskTick.set_iterator(taskScheduler.end());
-  m_taskTick.set_slot(sigc::mem_fun(*this, &Manager::receive_tick));
+  m_taskTick.set_slot(rak::mem_fn(this, &Manager::receive_tick));
 
-  taskScheduler.insert(&m_taskTick, (Timer::cache()).round_seconds());
+  priority_queue_insert(&taskScheduler, &m_taskTick, cachedTime.round_seconds());
 
-  m_handshakeManager->slot_connected(rak::make_mem_fn(this, &Manager::receive_connection));
-  m_handshakeManager->slot_download_id(rak::make_mem_fn(this, &Manager::retrive_download_id));
+  m_handshakeManager->slot_connected(rak::make_mem_fun(this, &Manager::receive_connection));
+  m_handshakeManager->slot_download_id(rak::make_mem_fun(this, &Manager::retrive_download_id));
 
-  m_listen->slot_incoming(rak::make_mem_fn(m_handshakeManager, &HandshakeManager::add_incoming));
+  m_listen->slot_incoming(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_incoming));
 }
 
 Manager::~Manager() {
-  taskScheduler.erase(&m_taskTick);
+  priority_queue_erase(&taskScheduler, &m_taskTick);
 
   m_handshakeManager->clear();
   m_downloadManager->clear();
@@ -89,6 +92,9 @@ Manager::~Manager() {
   delete m_hashQueue;
   delete m_listen;
   delete m_resourceManager;
+
+  delete m_uploadThrottle;
+  delete m_downloadThrottle;
 }
 
 void
@@ -98,11 +104,14 @@ Manager::initialize_download(DownloadWrapper* d) {
   d->set_file_manager(m_fileManager);
 
   m_downloadManager->insert(d);
-  m_resourceManager->insert(1, d->main());
+  m_resourceManager->insert(d->main(), 1);
 
-  d->main()->choke_manager()->slot_choke(rak::make_mem_fn(manager->resource_manager(), &ResourceManager::receive_choke));
-  d->main()->choke_manager()->slot_unchoke(rak::make_mem_fn(manager->resource_manager(), &ResourceManager::receive_unchoke));
-  d->main()->choke_manager()->slot_can_unchoke(rak::make_mem_fn(manager->resource_manager(), &ResourceManager::retrieve_can_unchoke));
+  d->main()->set_upload_throttle(m_uploadThrottle->throttle_list());
+  d->main()->set_download_throttle(m_downloadThrottle->throttle_list());
+
+  d->main()->choke_manager()->slot_choke(rak::make_mem_fun(manager->resource_manager(), &ResourceManager::receive_choke));
+  d->main()->choke_manager()->slot_unchoke(rak::make_mem_fun(manager->resource_manager(), &ResourceManager::receive_unchoke));
+  d->main()->choke_manager()->slot_can_unchoke(rak::make_mem_fun(manager->resource_manager(), &ResourceManager::retrieve_can_unchoke));
 }
 
 void
@@ -115,24 +124,39 @@ void
 Manager::receive_tick() {
   m_ticks++;
 
+  m_resourceManager->receive_tick();
+
   if (m_ticks % 4 == 0)
     std::for_each(m_downloadManager->begin(), m_downloadManager->end(), std::mem_fun(&DownloadWrapper::receive_keepalive));
 
-  m_resourceManager->receive_tick();
+  std::for_each(m_downloadManager->begin(), m_downloadManager->end(), std::mem_fun(&DownloadWrapper::receive_tick));
 
   // If you change the interval, make sure the above keepalive gets
   // triggered every 120 seconds.
-  taskScheduler.insert(&m_taskTick, (Timer::cache() + 30 * 1000000).round_seconds());
+  priority_queue_insert(&taskScheduler, &m_taskTick, (cachedTime + 30 * 1000000).round_seconds());
 }
 
 void
 Manager::receive_connection(SocketFd fd, const std::string& hash, const PeerInfo& peer) {
   DownloadManager::iterator itr = m_downloadManager->find(hash);
   
-  if (itr == m_downloadManager->end() ||
-      !(*itr)->main()->is_active() ||
-      !(*itr)->main()->connection_list()->insert((*itr)->main(), peer, fd))
+  if (itr == m_downloadManager->end()) {
     socketManager.close(fd);
+    return;
+  }
+
+  if (!peer.get_socket_address().is_valid()) {
+    (*itr)->main()->signal_network_log().emit("Caught a connection with invalid socket address.");
+
+    socketManager.close(fd);
+    return;
+  }    
+
+  if (!(*itr)->main()->is_active() ||
+      !(*itr)->main()->connection_list()->insert((*itr)->main(), peer, fd)) {
+    socketManager.close(fd);
+    return;
+  }
 }
 
 std::string
