@@ -37,11 +37,12 @@
 #ifndef LIBTORRENT_MANAGER_H
 #define LIBTORRENT_MANAGER_H
 
+#include <list>
 #include <string>
+#include <rak/priority_queue_default.h>
 
 #include "net/socket_address.h"
 #include "net/socket_fd.h"
-#include "utils/task_item.h"
 
 namespace torrent {
 
@@ -53,6 +54,9 @@ class DownloadWrapper;
 class FileManager;
 class ResourceManager;
 class PeerInfo;
+class ThrottleManager;
+
+typedef std::list<std::string> EncodingList;
 
 class Manager {
 public:
@@ -66,10 +70,13 @@ public:
   Listen*             listen()                                  { return m_listen; }
   ResourceManager*    resource_manager()                        { return m_resourceManager; }
 
-  SocketAddress&      get_local_address()                       { return m_localAddress; }
-  
-  const std::string&  get_bind_address()                        { return m_bindAddress; }
-  void                set_bind_address(const std::string& addr) { m_bindAddress = addr; }
+  SocketAddress*      local_address()                           { return &m_localAddress; }
+  SocketAddress*      bind_address()                            { return &m_bindAddress; }
+
+  EncodingList*       encoding_list()                           { return &m_encodingList; }
+
+  ThrottleManager*    upload_throttle()                         { return m_uploadThrottle; }
+  ThrottleManager*    download_throttle()                       { return m_downloadThrottle; }
 
   void                initialize_download(DownloadWrapper* d);
   void                cleanup_download(DownloadWrapper* d);
@@ -81,7 +88,7 @@ public:
 
 private:
   SocketAddress       m_localAddress;
-  std::string         m_bindAddress;
+  SocketAddress       m_bindAddress;
 
   DownloadManager*    m_downloadManager;
   FileManager*        m_fileManager;
@@ -90,8 +97,13 @@ private:
   Listen*             m_listen;
   ResourceManager*    m_resourceManager;
 
+  EncodingList        m_encodingList;
+
+  ThrottleManager*    m_uploadThrottle;
+  ThrottleManager*    m_downloadThrottle;
+
   unsigned int        m_ticks;
-  TaskItem            m_taskTick;
+  rak::priority_item  m_taskTick;
 };
 
 extern Manager* manager;
