@@ -42,28 +42,32 @@
 #include "data/piece.h"
 #include "net/manager.h"
 #include "net/socket_stream.h"
-#include "utils/bitfield_ext.h"
-#include "utils/task.h"
-#include "utils/throttle.h"
+#include "net/throttle_node.h"
+#include "globals.h"
 #include "torrent/rate.h"
 
 #include "peer_info.h"
+#include "peer_chunks.h"
 #include "protocol_base.h"
 #include "request_list.h"
 
 namespace torrent {
 
 // Base class for peer connection classes. Rename to PeerConnection
-// when the migration is complete.
+// when the migration is complete?
+//
+// This should really be modularized abit, there's too much stuff in
+// PeerConnectionBase and its children. Do we use additional layers of
+// inheritance or member instances?
 
 class DownloadMain;
 
 class PeerConnectionBase : public SocketStream {
 public:
-  typedef std::list<Piece>    PieceList;
-  typedef std::list<uint32_t> HaveQueue;
-  typedef ProtocolBase        ProtocolRead;
-  typedef ProtocolBase        ProtocolWrite;
+  typedef std::list<Piece>       PieceList;
+  typedef std::list<uint32_t>    HaveQueue;
+  typedef ProtocolBase           ProtocolRead;
+  typedef ProtocolBase           ProtocolWrite;
 
   // Find an optimal number for this.
   static const uint32_t read_size = 64;
@@ -71,7 +75,7 @@ public:
   PeerConnectionBase();
   virtual ~PeerConnectionBase();
   
-  bool                initialize(DownloadMain* download, const PeerInfo& p, SocketFd fd);
+  void                initialize(DownloadMain* download, const PeerInfo& p, SocketFd fd);
 
   bool                is_up_choked()                { return m_up->choked(); }
   bool                is_up_interested()            { return m_up->interested(); }
@@ -80,36 +84,24 @@ public:
 
   bool                is_upload_wanted() const      { return m_down->interested() && !m_snubbed; }
 
-  bool                is_down_throttled()           { return m_downThrottle != throttleRead.end(); }
-  bool                is_up_throttled()             { return m_upThrottle != throttleWrite.end(); }
-
   bool                is_snubbed() const            { return m_snubbed; }
-  bool                is_seeder() const             { return m_bitfield.all_set(); }
+  bool                is_seeder() const             { return m_peerChunks.bitfield()->all_set(); }
 
   const PeerInfo&     get_peer() const              { return m_peer; }
+  PeerChunks*         peer_chunks()                 { return &m_peerChunks; }
 
-  Rate&               peer_rate()                   { return m_peerRate; }
-  Rate&               up_rate()                     { return m_upRate; }
-  Rate&               down_rate()                   { return m_downRate; }
+  Rate*               peer_rate()                   { return &m_peerRate; }
+  Rate*               up_rate()                     { return m_upThrottle->rate(); }
+  Rate*               down_rate()                   { return m_downThrottle->rate(); }
 
   RequestList&        get_request_list()            { return m_requestList; }
   PieceList&          get_send_list()               { return m_sendList; }
 
-  const BitFieldExt&  bitfield() const          { return m_bitfield; }
-
   // Make sure you choke the peer when snubbing. Snubbing a peer will
   // only cause it not to be unchoked.
   void                set_snubbed(bool v);
 
-  uint32_t            pipe_size() const;
-
-  Timer               time_last_choked() const      { return m_timeLastChoked; }
-
-  void                insert_down_throttle();
-  void                remove_down_throttle();
-
-  void                insert_up_throttle();
-  void                remove_up_throttle();
+  rak::timer          time_last_choked() const      { return m_timeLastChoked; }
 
   // These must be implemented by the child class.
   virtual void        initialize_custom() = 0;
@@ -140,7 +132,6 @@ protected:
   void                read_buffer_move_unused();
 
   void                write_prepare_piece();
-  void                write_finished_piece();
 
   bool                read_bitfield_body();
   bool                read_bitfield_from_buffer(uint32_t msgLength);
@@ -160,23 +151,26 @@ protected:
   void                set_remote_interested();
   void                set_remote_not_interested();
 
+  // Insert into the poll unless we're blocking for throttling etc.
+  void                read_insert_poll_safe();
+  void                write_insert_poll_safe();
+
   DownloadMain*       m_download;
 
   ProtocolRead*       m_down;
   ProtocolWrite*      m_up;
 
   PeerInfo            m_peer;
+  PeerChunks          m_peerChunks;
   Rate                m_peerRate;
 
-  Rate                m_downRate;
-  ThrottlePeerNode    m_downThrottle;
+  ThrottleNode*       m_downThrottle;
   Piece               m_downPiece;
   ChunkHandle         m_downChunk;
 
   uint32_t            m_downStall;
 
-  Rate                m_upRate;
-  ThrottlePeerNode    m_upThrottle;
+  ThrottleNode*       m_upThrottle;
   Piece               m_upPiece;
   ChunkHandle         m_upChunk;
 
@@ -188,40 +182,11 @@ protected:
   bool                m_sendInterested;
 
   bool                m_snubbed;
-  BitFieldExt         m_bitfield;
-  Timer               m_timeLastChoked;
+  rak::timer          m_timeLastChoked;
 
-  Timer               m_timeLastRead;
+  rak::timer          m_timeLastRead;
 };
 
-inline void
-PeerConnectionBase::insert_down_throttle() {
-  if (!is_down_throttled())
-    m_downThrottle = throttleRead.insert(PeerConnectionThrottle(this, &PeerConnectionBase::receive_throttle_down_activate));
-}
-
-inline void
-PeerConnectionBase::remove_down_throttle() {
-  if (is_down_throttled()) {
-    throttleRead.erase(m_downThrottle);
-    m_downThrottle = throttleRead.end();
-  }
-}
-
-inline void
-PeerConnectionBase::insert_up_throttle() {
-  if (!is_up_throttled())
-    m_upThrottle = throttleWrite.insert(PeerConnectionThrottle(this, &PeerConnectionBase::receive_throttle_up_activate));
-}
-
-inline void
-PeerConnectionBase::remove_up_throttle() {
-  if (is_up_throttled()) {
-    throttleWrite.erase(m_upThrottle);
-    m_upThrottle = throttleWrite.end();
-  }
-}
-
 inline bool
 PeerConnectionBase::read_remaining() {
   m_down->buffer()->move_position(read_stream_throws(m_down->buffer()->position(),
@@ -238,6 +203,22 @@ PeerConnectionBase::write_remaining() {
   return !m_up->buffer()->remaining();
 }
 
+inline void
+PeerConnectionBase::read_insert_poll_safe() {
+  if (m_down->get_state() != ProtocolRead::IDLE)
+    return;
+
+  pollCustom->insert_read(this);
+}
+
+inline void
+PeerConnectionBase::write_insert_poll_safe() {
+  if (m_up->get_state() != ProtocolWrite::IDLE)
+    return;
+
+  pollCustom->insert_write(this);
+}
+
 }
 
 #endif
