@@ -41,6 +41,7 @@
 
 #include "data/content.h"
 #include "data/chunk_list_node.h"
+#include "download/chunk_selector.h"
 #include "download/download_main.h"
 
 #include "peer_connection_leech.h"
@@ -49,9 +50,9 @@ namespace torrent {
 
 PeerConnectionLeech::~PeerConnectionLeech() {
 //   if (m_download != NULL && m_down->get_state() != ProtocolRead::READ_BITFIELD)
-//     m_download->bitfield_counter().dec(m_bitfield.bitfield());
+//     m_download->bitfield_counter().dec(m_peerChunks.bitfield()->bitfield());
 
-//   taskScheduler.erase(&m_taskSendChoke);
+//   priority_queue_erase(&taskScheduler, &m_taskSendChoke);
 }
 
 void
@@ -67,7 +68,8 @@ PeerConnectionLeech::initialize_custom() {
 
 void
 PeerConnectionLeech::update_interested() {
-  if (m_download->delegator()->get_select().interested(m_bitfield.bitfield())) {
+// FIXME:   if (m_download->delegator()->get_select().interested(m_peerChunks.bitfield()->bitfield())) {
+  if (true) {
     m_sendInterested = !m_up->interested();
     m_up->set_interested(true);
   } else {
@@ -88,7 +90,7 @@ PeerConnectionLeech::receive_finished_chunk(int32_t index) {
 
 bool
 PeerConnectionLeech::receive_keepalive() {
-  if (Timer::cache() - m_timeLastRead > 240 * 1000000)
+  if (cachedTime - m_timeLastRead > 240 * 1000000)
     return false;
 
   // There's no point in adding ourselves to the write poll if the
@@ -96,8 +98,8 @@ PeerConnectionLeech::receive_keepalive() {
   if (m_up->get_state() == ProtocolWrite::IDLE &&
       m_up->can_write_keepalive()) {
 
+    write_insert_poll_safe();
     m_up->write_keepalive();
-    pollCustom->insert_write(this);
   }
 
   m_tryRequest = true;
@@ -162,16 +164,16 @@ PeerConnectionLeech::read_message() {
     m_down->set_choked(true);
 
     m_requestList.cancel();
-    remove_down_throttle();
+    m_download->download_throttle()->erase(m_downThrottle);
 
     return true;
 
   case ProtocolBase::UNCHOKE:
     if (is_down_choked()) {
+      write_insert_poll_safe();
+
       m_down->set_choked(false);
       m_tryRequest = true;
-
-      pollCustom->insert_write(this);
     }
 
     return true;
@@ -206,8 +208,8 @@ PeerConnectionLeech::read_message() {
       break;
 
     if (!m_up->choked()) {
+      write_insert_poll_safe();
       read_request_piece(m_down->read_request());
-      pollCustom->insert_write(this);
 
     } else {
       m_down->read_request();
@@ -238,7 +240,7 @@ PeerConnectionLeech::read_message() {
       
     } else if (down_chunk_from_buffer()) {
       // Done with chunk.
-      m_downChunk->set_time_modified(Timer::cache());
+      m_downChunk->set_time_modified(cachedTime);
       m_requestList.finished();
     
       if (m_downStall > 0)
@@ -247,13 +249,13 @@ PeerConnectionLeech::read_message() {
       // TODO: clear m_down.data?
       // TODO: remove throttle if choked? Rarely happens though.
       m_tryRequest = true;
-      pollCustom->insert_write(this);
+      write_insert_poll_safe();
 
       return true;
 
     } else {
       m_down->set_state(ProtocolRead::READ_PIECE);
-      insert_down_throttle();
+      m_download->download_throttle()->insert(m_downThrottle);
       return false;
     }
 
@@ -275,7 +277,7 @@ PeerConnectionLeech::read_message() {
 
 void
 PeerConnectionLeech::event_read() {
-  m_timeLastRead = Timer::cache();
+  m_timeLastRead = cachedTime;
 
   // Need to make sure ProtocolBuffer::end() is pointing to the end of
   // the unread data, and that the unread data starts from the
@@ -323,7 +325,7 @@ PeerConnectionLeech::event_read() {
 	if (!down_chunk())
 	  return;
 
-	m_downChunk->set_time_modified(Timer::cache());
+	m_downChunk->set_time_modified(cachedTime);
 	m_requestList.finished();
 	
 	if (m_downStall > 0)
@@ -333,7 +335,7 @@ PeerConnectionLeech::event_read() {
 	// TODO: remove throttle if choked? Rarely happens though.
 	m_tryRequest = true;
 	m_down->set_state(ProtocolRead::IDLE);
-	pollCustom->insert_write(this);
+	write_insert_poll_safe();
 	
 	break;
 
@@ -396,34 +398,37 @@ PeerConnectionLeech::event_read() {
 inline void
 PeerConnectionLeech::fill_write_buffer() {
   // No need to use delayed choke as we are a leecher.
-  if (m_sendChoked) {
+  if (m_sendChoked && m_up->can_write_choke()) {
     m_sendChoked = false;
     m_up->write_choke(m_up->choked());
 
     if (m_up->choked()) {
-      remove_up_throttle();
+      m_download->upload_throttle()->erase(m_upThrottle);
       up_chunk_release();
-	
       m_sendList.clear();
 
     } else {
-      insert_up_throttle();
+      m_download->upload_throttle()->insert(m_upThrottle);
     }
   }
 
+  // Send the interested state before any requests as some clients,
+  // e.g. BitTornado 0.7.14 and uTorrent 0.3.0, disconnect if a
+  // request has been received while uninterested. The problem arises
+  // as they send unchoke before receiving interested.
+  if (m_sendInterested && m_up->can_write_interested()) {
+    m_up->write_interested(m_up->interested());
+    m_sendInterested = false;
+  }
+
   if (m_tryRequest &&
 
       !(m_tryRequest = !should_request()) &&
       !(m_tryRequest = try_request_pieces()) &&
 
       !m_requestList.is_interested_in_active()) {
-    m_sendInterested = true;
-    m_up->set_interested(false);
-  }
-
-  if (m_sendInterested) {
-    m_up->write_interested(m_up->interested());
-    m_sendInterested = false;
+//     m_sendInterested = true;
+//     m_up->set_interested(false);
   }
 
   while (!m_haveQueue.empty() && m_up->can_write_have()) {
@@ -487,7 +492,6 @@ PeerConnectionLeech::event_write() {
 	if (!up_chunk())
 	  return;
 
-	write_finished_piece();
 	m_up->set_state(ProtocolWrite::IDLE);
 
 	break;
@@ -541,16 +545,16 @@ PeerConnectionLeech::event_write() {
 
 void
 PeerConnectionLeech::read_have_chunk(uint32_t index) {
-  if (index >= m_bitfield.size_bits())
+  if (index >= m_peerChunks.bitfield()->size_bits())
     throw network_error("Peer sent HAVE message with out-of-range index.");
 
-  if (m_bitfield.get(index))
+  if (m_peerChunks.bitfield()->get(index))
     return;
 
-  m_bitfield.set(index, true);
+  m_peerChunks.bitfield()->set(index, true);
   m_peerRate.insert(m_download->content()->chunk_size());
 
-  if (m_bitfield.all_set())
+  if (m_peerChunks.bitfield()->all_set())
     if (m_download->content()->is_done())
       throw close_connection();
     else
@@ -561,40 +565,41 @@ PeerConnectionLeech::read_have_chunk(uint32_t index) {
 
   if (is_up_interested()) {
 
-    if (!m_tryRequest && m_download->delegator()->get_select().interested(index)) {
+    if (!m_tryRequest && true) { // FIXME: m_download->delegator()->get_select().interested(index)) {
       m_tryRequest = true;
-      pollCustom->insert_write(this);
+      write_insert_poll_safe();
     }
 
   } else {
 
-    if (m_download->delegator()->get_select().interested(index)) {
+// FIXME:    if (m_download->delegator()->get_select().interested(index)) {
+    if (true) {
       m_sendInterested = true;
       m_up->set_interested(true);
       
       // Is it enough to insert into write here? Make the interested
       // check branch to include insert_write, even when not sending
       // interested.
-      pollCustom->insert_write(this);
+      write_insert_poll_safe();
     }
   }
 }
 
 void
 PeerConnectionLeech::finish_bitfield() {
-  m_bitfield.update_count();
+  m_peerChunks.bitfield()->update_count();
 
-  if (m_download->content()->is_done() && m_bitfield.all_set())
+  if (m_download->content()->is_done() && m_peerChunks.bitfield()->all_set())
     throw close_connection();
 
-  if (!m_download->content()->is_done() && m_download->delegator()->get_select().interested(m_bitfield.bitfield())) {
+  m_download->chunk_selector()->insert_peer_chunks(&m_peerChunks);
+
+  if (!m_download->content()->is_done() && true) { // FIXME: m_download->delegator()->get_select().interested(m_peerChunks.bitfield()->bitfield())) {
     m_sendInterested = true;
     m_up->set_interested(true);
   }
 
-//   m_download->bitfield_counter().inc(m_bitfield.bitfield());
-
-  pollCustom->insert_write(this);
+  write_insert_poll_safe();
 }
 
 bool
