@@ -43,15 +43,27 @@
 #include "download/delegator.h"
 #include "download/delegator_chunk.h"
 #include "download/delegator_reservee.h"
-#include "utils/bitfield.h"
 
+#include "peer_chunks.h"
 #include "request_list.h"
 
 namespace torrent {
 
+struct request_list_same_piece {
+  request_list_same_piece(const Piece& p) : m_piece(p) {}
+
+  bool operator () (const DelegatorReservee* d) {
+    return
+      m_piece.get_index() == d->get_piece().get_index() &&
+      m_piece.get_offset() == d->get_piece().get_offset();
+  }
+
+  Piece m_piece;
+};
+
 const Piece*
 RequestList::delegate() {
-  DelegatorReservee* r = m_delegator->delegate(*m_bitfield, m_affinity);
+  DelegatorReservee* r = m_delegator->delegate(m_peerChunks, m_affinity);
 
   if (r) {
     m_affinity = r->get_piece().get_index();
@@ -88,21 +100,24 @@ RequestList::stall() {
 bool
 RequestList::downloading(const Piece& p) {
   if (m_downloading)
-    throw internal_error("RequestList::downloading(...) bug, m_downloaing is already set");
+    throw internal_error("RequestList::downloading(...) bug, m_downloading is already set");
 
   remove_invalid();
 
   ReserveeList::iterator itr =
     std::find_if(m_reservees.begin(), m_reservees.end(),
-		 rak::equal(p, std::mem_fun(&DelegatorReservee::get_piece)));
+		 //rak::equal(p, std::mem_fun(&DelegatorReservee::get_piece)));
+		 request_list_same_piece(p));
   
   if (itr == m_reservees.end()) {
     itr = std::find_if(m_canceled.begin(), m_canceled.end(),
-		       rak::equal(p, std::mem_fun(&DelegatorReservee::get_piece)));
+		       //rak::equal(p, std::mem_fun(&DelegatorReservee::get_piece)));
+		       request_list_same_piece(p));
 
     if (itr == m_canceled.end())
       return false;
 
+    // The iterator needs to remain valid.
     m_reservees.push_front(*itr);
     m_canceled.erase(itr);
 
@@ -110,13 +125,20 @@ RequestList::downloading(const Piece& p) {
     cancel_range(itr);
   }
   
+  // Make sure pieces are removed from the reservee list if the peer
+  // returns zero length pieces.
+  if (p.get_length() != (*itr)->get_piece().get_length()) {
+    m_reservees.erase(itr);
+    return false;
+  }
+
   m_downloading = true;
 
   if (p != m_reservees.front()->get_piece())
     throw internal_error("RequestList::downloading(...) did not add the new piece to the front of the list");
   
-  if ((*m_delegator->get_select().bitfield())[p.get_index()])
-    throw internal_error("RequestList::downloading(...) called with a piece index we already have");
+//   if ((*m_delegator->get_select().bitfield())[p.get_index()])
+//     throw internal_error("RequestList::downloading(...) called with a piece index we already have");
 
   return true;
 }
@@ -155,7 +177,7 @@ struct equals_reservee : public std::binary_function<DelegatorReservee*, int32_t
 bool
 RequestList::is_interested_in_active() const {
   for (Delegator::Chunks::const_iterator itr = m_delegator->get_chunks().begin(), last = m_delegator->get_chunks().end(); itr != last; ++itr)
-    if ((*m_bitfield)[(*itr)->get_index()])
+    if (m_peerChunks->bitfield()->get((*itr)->get_index()))
       return true;
 
   return false;
@@ -200,4 +222,38 @@ RequestList::remove_invalid() {
   return count;
 }
 
+uint32_t
+RequestList::calculate_pipe_size(uint32_t rate) {
+  // Change into KB.
+  rate /= 1024;
+
+  // Request enough to have enough for 16 seconds at fast rates.
+
+  if (!m_delegator->get_aggressive()) {
+    uint32_t queueKB = std::min(rate * 30, (uint32_t)16 << 10);
+  
+    return (queueKB / (Delegator::block_size / 1024)) + 2;
+
+  } else {
+    uint32_t queueKB = std::min(rate * 16, (uint32_t)16 << 10);
+  
+    return (queueKB / (Delegator::block_size / 1024)) + 1;
+  }
+}
+
+
+//   if ()
+//     if (s < 50000)
+//       return std::max((uint32_t)2, (s + 2000) / 2000);
+//     else
+//       return std::min((uint32_t)200, (s + 160000) / 4000);
+
+//   else
+//     if (s < 4000)
+//       return 1;
+//     else
+//       return std::min((uint32_t)80, (s + 32000) / 8000);
+
+// }
+
 }
