@@ -38,7 +38,7 @@
 
 #include <iomanip>
 #include <sstream>
-#include <sigc++/bind.h>
+#include <rak/string_manip.h>
 
 #include "torrent/exceptions.h"
 #include "torrent/http.h"
@@ -53,19 +53,11 @@ TrackerHttp::TrackerHttp(TrackerInfo* info, const std::string& url) :
   m_get(Http::call_factory()),
   m_data(NULL) {
 
-  m_get->set_user_agent(PACKAGE "/" VERSION);
-
   m_get->signal_done().connect(sigc::mem_fun(*this, &TrackerHttp::receive_done));
   m_get->signal_failed().connect(sigc::mem_fun(*this, &TrackerHttp::receive_failed));
-
-  m_taskTimeout.set_iterator(taskScheduler.end());
-  m_taskTimeout.set_slot(sigc::bind(sigc::mem_fun(*this, &TrackerHttp::receive_failed), "Connection stalled"));
 }
 
 TrackerHttp::~TrackerHttp() {
-  if (taskScheduler.is_scheduled(&m_taskTimeout))
-    throw internal_error("TrackerHttp::~TrackerHttp() called but m_taskTimeout still scheduled.");
-
   delete m_get;
   delete m_data;
 }
@@ -89,18 +81,13 @@ TrackerHttp::send_state(TrackerInfo::State state, uint64_t down, uint64_t up, ui
 
   std::stringstream s;
 
-  s << m_url << "?info_hash=";
-  escape_string(m_info->get_hash(), s);
+  s << m_url
+    << "?info_hash=" << rak::copy_escape_html(m_info->get_hash())
+    << "&peer_id=" << rak::copy_escape_html(m_info->get_local_id())
+    << "&key=" << std::hex << std::setw(8) << std::setfill('0') << m_info->get_key() << std::dec;
 
-  s << "&peer_id=";
-  escape_string(m_info->get_local_id(), s);
-
-  s << "&key=" << std::hex << std::setw(8) << std::setfill('0') << m_info->get_key() << std::dec;
-
-  if (!m_trackerId.empty()) {
-    s << "&trackerid=";
-    escape_string(m_trackerId, s);
-  }
+  if (!m_trackerId.empty())
+    s << "&trackerid=" << rak::copy_escape_html(m_trackerId);
 
   if (!m_info->get_local_address().is_address_any())
     s << "&ip=" << m_info->get_local_address().get_address();
@@ -134,10 +121,9 @@ TrackerHttp::send_state(TrackerInfo::State state, uint64_t down, uint64_t up, ui
 
   m_get->set_url(s.str());
   m_get->set_stream(m_data);
+  m_get->set_timeout(2 * 60);
 
   m_get->start();
-
-  taskScheduler.insert(&m_taskTimeout, (Timer::cache() + m_info->http_timeout() * 1000000).round_seconds());
 }
 
 void
@@ -150,8 +136,6 @@ TrackerHttp::close() {
 
   delete m_data;
   m_data = NULL;
-
-  taskScheduler.erase(&m_taskTimeout);
 }
 
 TrackerHttp::Type
@@ -159,23 +143,6 @@ TrackerHttp::get_type() const {
   return TRACKER_HTTP;
 }
 
-void
-TrackerHttp::escape_string(const std::string& src, std::ostream& stream) {
-  // TODO: Correct would be to save the state.
-  stream << std::hex << std::uppercase;
-
-  for (std::string::const_iterator itr = src.begin(); itr != src.end(); ++itr)
-    if ((*itr >= 'A' && *itr <= 'Z') ||
-	(*itr >= 'a' && *itr <= 'z') ||
-	(*itr >= '0' && *itr <= '9') ||
-	*itr == '-')
-      stream << *itr;
-    else
-      stream << '%' << ((unsigned char)*itr >> 4) << ((unsigned char)*itr & 0xf);
-
-  stream << std::dec << std::nouppercase;
-}
-
 void
 TrackerHttp::receive_done() {
   if (m_data == NULL)
@@ -192,27 +159,27 @@ TrackerHttp::receive_done() {
 
   if (b.has_key("failure reason"))
     return receive_failed("Failure reason \"" +
-			 (b["failure reason"].is_string() ?
-			  b["failure reason"].as_string() :
+			 (b.get_key("failure reason").is_string() ?
+			  b.get_key("failure reason").as_string() :
 			  std::string("failure reason not a string"))
 			 + "\"");
 
-  if (b.has_key("interval") && b["interval"].is_value())
-    m_slotSetInterval(b["interval"].as_value());
+  if (b.has_key("interval") && b.get_key("interval").is_value())
+    m_slotSetInterval(b.get_key("interval").as_value());
   
-  if (b.has_key("min interval") && b["min interval"].is_value())
-    m_slotSetMinInterval(b["min interval"].as_value());
+  if (b.has_key("min interval") && b.get_key("min interval").is_value())
+    m_slotSetMinInterval(b.get_key("min interval").as_value());
 
-  if (b.has_key("tracker id") && b["tracker id"].is_string())
-    m_trackerId = b["tracker id"].as_string();
+  if (b.has_key("tracker id") && b.get_key("tracker id").is_string())
+    m_trackerId = b.get_key("tracker id").as_string();
 
   AddressList l;
 
   try {
-    if (b["peers"].is_string())
-      parse_address_compact(l, b["peers"].as_string());
+    if (b.get_key("peers").is_string())
+      parse_address_compact(l, b.get_key("peers").as_string());
     else
-      parse_address_normal(l, b["peers"].as_list());
+      parse_address_normal(l, b.get_key("peers").as_list());
 
   } catch (bencode_error& e) {
     return receive_failed(e.what());
