@@ -47,14 +47,13 @@ TrackerManager::TrackerManager() :
   m_control(new TrackerControl),
   m_isRequesting(false),
   m_numRequests(0),
-  m_maxRequests(5),
+  m_maxRequests(3),
   m_initialTracker(0) {
 
-  m_control->slot_success(rak::make_mem_fn(this, &TrackerManager::receive_success));
-  m_control->slot_failed(rak::make_mem_fn(this, &TrackerManager::receive_failed));
+  m_control->slot_success(rak::make_mem_fun(this, &TrackerManager::receive_success));
+  m_control->slot_failed(rak::make_mem_fun(this, &TrackerManager::receive_failed));
 
-  m_taskTimeout.set_iterator(taskScheduler.end());
-  m_taskTimeout.set_slot(sigc::mem_fun(*this, &TrackerManager::receive_timeout));
+  m_taskTimeout.set_slot(rak::mem_fn(this, &TrackerManager::receive_timeout));
 }
 
 TrackerManager::~TrackerManager() {
@@ -66,7 +65,7 @@ TrackerManager::~TrackerManager() {
 
 bool
 TrackerManager::is_active() const {
-  return taskScheduler.is_scheduled(&m_taskTimeout) || m_control->is_busy();
+  return m_taskTimeout.is_queued() || m_control->is_busy();
 }
 
 bool
@@ -79,7 +78,7 @@ TrackerManager::close() {
   m_isRequesting = false;
 
   m_control->close();
-  taskScheduler.erase(&m_taskTimeout);
+  priority_queue_erase(&taskScheduler, &m_taskTimeout);
 }
 
 void
@@ -123,16 +122,17 @@ TrackerManager::send_completed() {
 // through this function has reached a certain limit, it will stop the
 // request. 'm_maxRequests' thus makes sure that a client with a very
 // high "min peers" setting will not cause too much traffic.
-void
+bool
 TrackerManager::request_current() {
-  if (m_control->is_busy() ||
-      m_numRequests >= m_maxRequests)
-    return;
+  if (m_control->is_busy() || m_numRequests >= m_maxRequests)
+    return false;
 
   // Keep track of how many times we've requested from the current
   // tracker without waiting for some minimum interval.
   m_isRequesting = true;
   manual_request(true);
+
+  return true;
 }
 
 void
@@ -142,6 +142,7 @@ TrackerManager::request_next() {
     return;
 
   m_isRequesting = true;
+  m_numRequests  = 0;
   manual_request(true);
 }
 
@@ -150,16 +151,16 @@ TrackerManager::request_next() {
 // affected.
 void
 TrackerManager::manual_request(bool force) {
-  if (!taskScheduler.is_scheduled(&m_taskTimeout))
+  if (!m_taskTimeout.is_queued())
     return;
 
-  Timer t(Timer::cache() + 2 * 1000000);
+  rak::timer t(cachedTime + 2 * 1000000);
   
   if (!force)
     t = std::max(t, m_control->time_last_connection() + m_control->get_min_interval() * 1000000);
 
-  taskScheduler.erase(&m_taskTimeout);
-  taskScheduler.insert(&m_taskTimeout, t.round_seconds());
+  priority_queue_erase(&taskScheduler, &m_taskTimeout);
+  priority_queue_insert(&taskScheduler, &m_taskTimeout, t.round_seconds());
 }
 
 void
@@ -177,6 +178,14 @@ TrackerManager::size() const {
   return m_control->get_list().size();
 }
 
+TrackerManager::size_type
+TrackerManager::group_size() const {
+  if (m_control->get_list().rbegin() == m_control->get_list().rend())
+    return 0;
+  else
+    return m_control->get_list().rbegin()->first + 1;
+}
+
 TrackerManager::value_type
 TrackerManager::get(size_type idx) const {
   return m_control->get_list()[idx];
@@ -233,7 +242,7 @@ TrackerManager::receive_success(AddressList* l) {
   m_isRequesting = false;
 
   m_control->set_state(TrackerInfo::NONE);
-  taskScheduler.insert(&m_taskTimeout, (Timer::cache() + m_control->get_normal_interval() * 1000000).round_seconds());
+  priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + m_control->get_normal_interval() * 1000000).round_seconds());
 
   m_slotSuccess(l);
 }
@@ -250,9 +259,9 @@ TrackerManager::receive_failed(const std::string& msg) {
       // Don't start from the beginning of the list if we've gone
       // through the whole list. Return to normal timeout.
       m_isRequesting = false;
-      taskScheduler.insert(&m_taskTimeout, (Timer::cache() + m_control->get_normal_interval() * 1000000).round_seconds());
+      priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + m_control->get_normal_interval() * 1000000).round_seconds());
     } else {
-      taskScheduler.insert(&m_taskTimeout, (Timer::cache() + 20 * 1000000).round_seconds());
+      priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + 20 * 1000000).round_seconds());
     }
 
   } else {
@@ -260,7 +269,7 @@ TrackerManager::receive_failed(const std::string& msg) {
     if (m_control->focus_index() == m_control->get_list().size())
       m_control->set_focus_index(0);
     
-    taskScheduler.insert(&m_taskTimeout, (Timer::cache() + 20 * 1000000).round_seconds());
+    priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + 20 * 1000000).round_seconds());
   }
 
   m_slotFailed(msg);
