@@ -41,7 +41,7 @@
 #include <rak/functional.h>
 
 #include "net/socket_address.h"
-#include "utils/task.h"
+#include "globals.h"
 
 namespace torrent {
 
@@ -56,8 +56,8 @@ public:
   typedef std::pair<int, TrackerBase*>            value_type;
   typedef std::list<SocketAddress>                AddressList;
 
-  typedef rak::mem_fn1<DownloadWrapper, void, AddressList*>       SlotSuccess;
-  typedef rak::mem_fn1<DownloadWrapper, void, const std::string&> SlotFailed;
+  typedef rak::mem_fun1<DownloadWrapper, void, AddressList*>       SlotSuccess;
+  typedef rak::mem_fun1<DownloadWrapper, void, const std::string&> SlotFailed;
 
   TrackerManager();
   ~TrackerManager();
@@ -75,7 +75,7 @@ public:
   // list. These functions will start from the current focus and
   // iterate to the next if it is unable to connect. Once the end is
   // reached it will stop.
-  void                request_current();
+  bool                request_current();
   void                request_next();
 
   void                manual_request(bool force);
@@ -84,6 +84,7 @@ public:
   void                randomize();
 
   size_type           size() const;
+  size_type           group_size() const;
 
   value_type          get(size_type idx) const;
   size_type           focus_index() const;
@@ -92,7 +93,7 @@ public:
 
   TrackerInfo*        tracker_info();
 
-  Timer               get_next_timeout() const;
+  rak::timer          get_next_timeout() const                  { return m_taskTimeout.time(); }
 
   void                slot_success(SlotSuccess s)               { m_slotSuccess = s; }
   void                slot_failed(SlotFailed s)                 { m_slotFailed = s; }
@@ -117,14 +118,9 @@ private:
   SlotSuccess         m_slotSuccess;
   SlotFailed          m_slotFailed;
 
-  TaskItem            m_taskTimeout;
+  rak::priority_item  m_taskTimeout;
 };
 
-inline Timer
-TrackerManager::get_next_timeout() const {
-  return taskScheduler.is_scheduled(&m_taskTimeout) ? m_taskTimeout.get_time() : Timer();
-}
-
 }
 
 #endif
