@@ -51,8 +51,7 @@ TrackerUdp::TrackerUdp(TrackerInfo* info, const std::string& url) :
   m_readBuffer(NULL),
   m_writeBuffer(NULL) {
 
-  m_taskTimeout.set_iterator(taskScheduler.end());
-  m_taskTimeout.set_slot(sigc::mem_fun(*this, &TrackerUdp::receive_timeout));
+  m_taskTimeout.set_slot(rak::mem_fn(this, &TrackerUdp::receive_timeout));
 }
 
 TrackerUdp::~TrackerUdp() {
@@ -95,7 +94,7 @@ TrackerUdp::send_state(TrackerInfo::State state,
   pollCustom->insert_error(this);
 
   m_tries = m_info->udp_tries();
-  taskScheduler.insert(&m_taskTimeout, (Timer::cache() + m_info->udp_timeout() * 1000000).round_seconds());
+  priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + m_info->udp_timeout() * 1000000).round_seconds());
 }
 
 void
@@ -109,7 +108,7 @@ TrackerUdp::close() {
   m_readBuffer = NULL;
   m_writeBuffer = NULL;
 
-  taskScheduler.erase(&m_taskTimeout);
+  priority_queue_erase(&taskScheduler, &m_taskTimeout);
 
   pollCustom->remove_read(this);
   pollCustom->remove_write(this);
@@ -133,13 +132,13 @@ TrackerUdp::receive_failed(const std::string& msg) {
 
 void
 TrackerUdp::receive_timeout() {
-  if (taskScheduler.is_scheduled(&m_taskTimeout))
+  if (m_taskTimeout.is_queued())
     throw internal_error("TrackerUdp::receive_timeout() called but m_taskTimeout is still scheduled.");
 
   if (--m_tries == 0) {
     receive_failed("Unable to connect to UDP tracker.");
   } else {
-    taskScheduler.insert(&m_taskTimeout, (Timer::cache() + m_info->udp_timeout() * 1000000).round_seconds());
+    priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + m_info->udp_timeout() * 1000000).round_seconds());
 
     pollCustom->insert_write(this);
   }
@@ -167,8 +166,8 @@ TrackerUdp::event_read() {
 
     prepare_announce_input();
 
-    taskScheduler.erase(&m_taskTimeout);
-    taskScheduler.insert(&m_taskTimeout, (Timer::cache() + m_info->udp_timeout() * 1000000).round_seconds());
+    priority_queue_erase(&taskScheduler, &m_taskTimeout);
+    priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + m_info->udp_timeout() * 1000000).round_seconds());
 
     m_tries = m_info->udp_tries();
     pollCustom->insert_write(this);
@@ -214,7 +213,7 @@ TrackerUdp::parse_url() {
   int port;
   char hostname[256];
       
-  if (std::sscanf(m_url.c_str(), "udp://%256[^:]:%i", hostname, &port) != 2 ||
+  if (std::sscanf(m_url.c_str(), "udp://%255[^:]:%i", hostname, &port) != 2 ||
       hostname[0] == '\0' ||
       port <= 0 || port >= (1 << 16))
     return false;
